@@ -38,8 +38,8 @@ import sipconfig
 
 
 # Initialise the globals.
-pyqt_version = 0x040701
-pyqt_version_str = "4.7.1-snapshot-20100120"
+pyqt_version = 0x040704
+pyqt_version_str = "snapshot-4.7.4-4e4296742920"
 
 sip_min_version = 0x040a00
 
@@ -327,7 +327,8 @@ class ConfigurePyQt4:
             0x040501: "Qt_4_5_0",
             0x040600: "Qt_4_5_1",
             0x040601: "Qt_4_6_0",
-            0x050000: "Qt_4_6_1"
+            0x040602: "Qt_4_6_1",
+            0x050000: "Qt_4_6_2"
         }
 
     def check_modules(self):
@@ -363,7 +364,7 @@ class ConfigurePyQt4:
         check_module("QtWebKit", "qwebpage.h", "new QWebPage()")
         check_module("QtXml", "qdom.h", "new QDomDocument()")
         check_module("QtXmlPatterns", "qxmlname.h", "new QXmlName()")
-        check_module("phonon", "Phonon/VideoWidget",
+        check_module("phonon", "phonon/videowidget.h",
                 "new Phonon::VideoWidget()")
         check_module("QtAssistant", "qassistantclient.h",
                 "new QAssistantClient(\"foo\")", extra_lib_dirs=ass_lib_dirs,
@@ -380,7 +381,7 @@ class ConfigurePyQt4:
         check_module("QAxContainer", "qaxobject.h", "new QAxObject()",
                 extra_libs=["QAxContainer"])
 
-        if os.path.isdir("dbus"):
+        if os.path.isdir(os.path.join(src_dir, "dbus")):
             check_dbus()
 
     def code(self):
@@ -391,10 +392,10 @@ class ConfigurePyQt4:
         sp_libs, sp_libdirs = self._static_plugins("QtCore")
         sp_incdirs = []
 
-        qpy_inc_dir, qpy_lib_dir = self._qpy_directories("QtCore")
+        qpy_inc_dir, qpy_lib_dir, qpy_lib = self._qpy_directories("QtCore", "qpycore")
         sp_incdirs.append(qpy_inc_dir)
         sp_libdirs.append(qpy_lib_dir)
-        sp_libs.append("qpycore")
+        sp_libs.append(qpy_lib)
 
         if opts.vendorcheck:
             sp_incdirs.append(opts.vendincdir)
@@ -415,10 +416,10 @@ class ConfigurePyQt4:
             sp_libs, sp_libdirs = self._static_plugins("QtGui")
             sp_incdirs = []
 
-            qpy_inc_dir, qpy_lib_dir = self._qpy_directories("QtGui")
+            qpy_inc_dir, qpy_lib_dir, qpy_lib = self._qpy_directories("QtGui", "qpygui")
             sp_incdirs.append(qpy_inc_dir)
             sp_libdirs.append(qpy_lib_dir)
-            sp_libs.append("qpygui")
+            sp_libs.append(qpy_lib)
 
             if opts.bigqt:
                 cons_xtra_incdirs.extend(sp_incdirs)
@@ -440,6 +441,7 @@ class ConfigurePyQt4:
             generate_code("QtNetwork")
 
         if "QtOpenGL" in pyqt_modules:
+            generate_OpenGL_extras()
             generate_code("QtOpenGL")
 
         if "QtScript" in pyqt_modules:
@@ -482,18 +484,17 @@ class ConfigurePyQt4:
             generate_code("QtAssistant")
 
         if "QtDesigner" in pyqt_modules:
-            qpy_inc_dir, qpy_lib_dir = self._qpy_directories("QtDesigner")
+            qpy_inc_dir, qpy_lib_dir, qpy_lib = self._qpy_directories("QtDesigner", "qpydesigner")
 
             if opts.bigqt:
                 cons_xtra_incdirs.append(qpy_inc_dir)
                 cons_xtra_libdirs.append(qpy_lib_dir)
-                cons_xtra_libs.append("qpydesigner")
+                cons_xtra_libs.append(qpy_lib)
 
                 generate_code("QtDesigner")
             else:
                 generate_code("QtDesigner", extra_include_dirs=[qpy_inc_dir],
-                        extra_lib_dirs=[qpy_lib_dir],
-                        extra_libs=["qpydesigner"])
+                        extra_lib_dirs=[qpy_lib_dir], extra_libs=[qpy_lib])
 
         if "QAxContainer" in pyqt_modules:
             generate_code("QAxContainer")
@@ -609,23 +610,26 @@ class ConfigurePyQt4:
 
         f.close()
 
-    def _qpy_directories(self, mname):
-        """Return a tuple of the directories containing the header files and
-        library for the support library for the given module.
+    def _qpy_directories(self, mname, lib_name):
+        """Return a 3-tuple of the directories containing the header files, the
+        directory containing the library, and the name of the support library
+        for the given module.
 
         mname is the name of the module.
+        lib_name is the normal name of the support library.
         """
         qpy_dir = os.path.join("qpy", mname)
 
         if sys.platform == 'win32':
             if opts.debug:
                 qpy_lib_dir = os.path.join(qpy_dir, "debug")
+                lib_name = 'd' + lib_name
             else:
                 qpy_lib_dir = os.path.join(qpy_dir, "release")
         else:
             qpy_lib_dir = qpy_dir
 
-        return os.path.join(src_dir, qpy_dir), os.path.abspath(qpy_lib_dir)
+        return os.path.join(src_dir, qpy_dir), os.path.abspath(qpy_lib_dir), lib_name
 
     def _static_plugins(self, mname):
         """Return a tuple of the libraries (in platform neutral format) and the
@@ -1065,7 +1069,10 @@ def run_command(cmd):
     lout = fout.readline()
     while lout:
         if opts.verbose:
-            sys.stdout.write(str(lout))
+            if sys.hexversion >= 0x03000000:
+                sys.stdout.write(str(lout, encoding=sys.stdout.encoding))
+            else:
+                sys.stdout.write(lout)
 
         lout = fout.readline()
 
@@ -1088,6 +1095,73 @@ def remove_file(fname):
         pass
 
 
+def generate_OpenGL_extras():
+    """Generate the extras needed by the QtOpenGL module (i.e. the .sip file
+    defining the correct typedefs for the OpenGL data types.
+    """
+    sipconfig.inform("Determining the OpenGL data types...")
+
+    src = "opengl_extras.cpp"
+
+    f = open(src, "w")
+
+    f.write(
+"""#include <QFile>
+#include <QTextStream>
+#include <qgl.h>
+
+int main(int, char **)
+{
+    QFile outf("./sip/QtOpenGL/opengl_types.sip");
+
+    if (!outf.open(QIODevice::WriteOnly|QIODevice::Truncate|QIODevice::Text))
+        return 1;
+
+    QTextStream out(&outf);
+
+    if (sizeof (long) == sizeof (GLint))
+        out << "typedef long GLint;\\n";
+    else
+        out << "typedef int GLint;\\n";
+
+    if (sizeof (unsigned long) == sizeof (GLuint))
+        out << "typedef unsigned long GLuint;\\n";
+    else
+        out << "typedef unsigned GLuint;\\n";
+
+    if (sizeof (unsigned long) == sizeof (GLenum))
+        out << "typedef unsigned long GLenum;\\n";
+    else
+        out << "typedef unsigned GLenum;\\n";
+
+    if (sizeof (unsigned long) == sizeof (GLbitfield))
+        out << "typedef unsigned long GLbitfield;\\n";
+    else
+        out << "typedef unsigned GLbitfield;\\n";
+
+    out << "typedef float GLfloat;\\n";
+
+    return 0;
+}
+""")
+
+    f.close()
+
+    cmd = compile_qt_program(src, "QtOpenGL")
+
+    if cmd is None:
+        sipconfig.error("Unable to determine the OpenGL data types.")
+
+    # SIP's build system (specifically ProgramMakefile.build_command()) assumes
+    # that an executable can be created by running a single command.  In the
+    # case of MSVC this is incorrect.  As a quick hack we do the extra step
+    # here if it looks like it is needed.
+    if os.access("opengl_extras.manifest", os.F_OK):
+        run_command("mt -nologo -manifest opengl_extras.manifest -outputresource:opengl_extras.exe;1")
+
+    run_command(cmd)
+
+
 def compile_qt_program(name, mname, extra_include_dirs=None, extra_lib_dirs=None, extra_libs=None):
     """Compile a simple Qt application.
 
@@ -1103,7 +1177,7 @@ def compile_qt_program(name, mname, extra_include_dirs=None, extra_lib_dirs=None
     opengl = (mname == "QtOpenGL")
 
     qt = [mname]
-    if mname == "QtWebKit":
+    if mname in ("QtOpenGL", "QtWebKit"):
         qt.append("QtCore")
 
     makefile = sipconfig.ProgramMakefile(sipcfg, console=1, qt=qt, warnings=0,
@@ -1840,6 +1914,10 @@ int main(int, char **)
     out << "PyQt_NoPrintRangeBug\\n";
 #endif
 
+#if defined(QT_OPENGL_ES)
+    out << "PyQt_NoOpenGLES\\n";
+#endif
+
     if (sizeof (qreal) != sizeof (double))
         out << "PyQt_qreal_double\\n";
 
